//------------------------------------------------------------------------------
//
//   Greg's Blocks - BlockShape
//
//------------------------------------------------------------------------------

package gcewing.blocks;

import java.util.List;
import net.minecraft.src.*;

public class BlockShape extends BlockContainer {

	public static int modelID;
	public static TEShape lastPlacedTileEntity;

	public BlockShape(int id) {
		super(id, 0, Material.rock);
		//setLightOpacity(255);
	}

	@Override
	public void onBlockPlacedBy(World world, int x, int y, int z, EntityLiving player) {
		int angle = Utils.placementRotation(player);
		world.setBlockMetadataWithNotify(x, y, z, angle);
		lastPlacedTileEntity = (TEShape)world.getBlockTileEntity(x, y, z);
		System.out.printf("BlockShape.onBlockPlacedBy: tile entity = %s\n", lastPlacedTileEntity);
	}
	
	@Override
	public TileEntity createNewTileEntity(World world) {
		return new TEShape();
	}

	@Override
	public boolean renderAsNormalBlock() {
		return false;
	}
	
	@Override
	public int getRenderType() {
		return modelID;
	}
	
	@Override
	public boolean isOpaqueCube() {
		return false;
	}
	
	@Override
	public void addCollidingBlockToList(World world, int x, int y, int z, 
		AxisAlignedBB clip, List list, Entity entity)
	{
		int data = world.getBlockMetadata(x, y, z);
		TEShape te = (TEShape)world.getBlockTileEntity(x, y, z);
		Matrix3 rot = Matrix3.blockRotations[data];
		Vector3 org = new Vector3(x + 0.5, y + 0.5, z + 0.5);
		addBox(-0.5, 0.5,   -0.5, 0.0,  -0.5, 0.5,   rot, org, clip, list);
		if (te.shape == 0 || te.shape == 2)
			addBox(-0.5, 0.5,   0.0, 0.5,   0.0, 0.5,   rot, org, clip, list);
		if (te.shape == 1)
			addBox(-0.5, 0.0,   0.0, 0.5,   0.0, 0.5,   rot, org, clip, list);
		if (te.shape == 2)
			addBox(-0.5, 0.0,   0.0, 0.5,   -0.5, 0.0,  rot, org, clip, list);
	}

	void addBox(double x0, double x1, double y0, double y1, double z0, double z1,
			Matrix3 rot, Vector3 org, AxisAlignedBB clip, List list)
	{
		Vector3 p0 = rot.mul(x0, y0, z0).add(org);
		Vector3 p1 = rot.mul(x1, y1, z1).add(org);
		if (p0.x < p1.x) {x0 = p0.x; x1 = p1.x;} else {x0 = p1.x; x1 = p0.x;}
		if (p0.y < p1.y) {y0 = p0.y; y1 = p1.y;} else {y0 = p1.y; y1 = p0.y;}
		if (p0.z < p1.z) {z0 = p0.z; z1 = p1.z;} else {z0 = p1.z; z1 = p0.z;}
		AxisAlignedBB box = AxisAlignedBB.getAABBPool().addOrModifyAABBInPool(x0, y0, z0, x1, y1, z1);
		//System.out.printf("BlockShape.addBox: %s\n", box);
		if (box != null && clip.intersectsWith(box))
			list.add(box);
	}
	
	public boolean canHarvestBlock(EntityPlayer player, int meta) {
		// Item drop occurs in onBlockRemoval
		System.out.printf("BlockShape.canHarvestBlock: by %s\n", player);
		return false;
	}

//	public ArrayList<ItemStack> getBlockDropped(World world, int x, int y, int z, int metadata, int fortune) {
//		//ArrayList<ItemStack> result = super.getBlockDropped(world, x, y, z, metadata, fortune);
//		ArrayList<ItemStack> result = new ArrayList<ItemStack>();
//		ItemStack stack = new ItemStack(this, 1);
//		TileEntity te = world.getBlockTileEntity(x, y, z);
//		stack.setTagCompound(Utils.nbtFromTileEntity(te));
//		result.add(stack);
//		System.out.printf("BlockShape.getBlockDropped --> %s\n", result);
//		return result;
//	}

	@Override
	public void onBlockHarvested(World world, int x, int y, int z, int data, EntityPlayer player) {
		//System.out.printf("BlockShape.onBlockHarvested: player = %s\n", player);
		if (!Utils.playerIsInCreativeMode(player)) {
			TileEntity te = world.getBlockTileEntity(x, y, z);
			ItemStack stack = Utils.blockStackWithTileEntity(this, 1, te);
			dropBlockAsItem_do(world, x, y, z, stack);
		}
	}

//	public float blockStrength(World world, EntityPlayer player, int x, int y, int z) {

	@Override
	public float getPlayerRelativeBlockHardness(EntityPlayer player, World world, int x, int y, int z) {
		TEShape te = (TEShape)world.getBlockTileEntity(x, y, z);
		Block base = te.getBaseBlock();
		return base.getPlayerRelativeBlockHardness(player, world, x, y, z);
	}

	@Override
	public void setBlockBoundsBasedOnState(IBlockAccess world, int x, int y, int z) {
		TEShape te = (TEShape)world.getBlockTileEntity(x, y, z);
		if (te.shape <= 2)
			setBlockBounds(0.0F, 0.0F, 0.0F, 1.0F, 1.0F, 1.0F);
		else
			setBlockBounds(0.0F, 0.0F, 0.0F, 1.0F, 0.5F, 1.0F);
	}

}
